/*------------------------------------------------------------------------
    File        : WebApp.EC.BL.Container.cls
    Author(s)   : marc
    Created     : 31-08-2018 16:56:54
    Notes       : Customized BE instance with SPJFP features available.
  ----------------------------------------------------------------------*/
@program FILE(name="ContainerBE.cls", module="AppServer").
@openapi.openedge.export FILE(type="REST", executionMode="singleton", useReturnValue="false", writeDataSetBeforeImage="false").
@progress.service.resource FILE(name="WebApp.EC.BL.Container", URI="/EC/BL/ContainerBE", schemaName="dsContainer", schemaFile="").

USING Progress.Lang.*.
USING Progress.Json.ObjectModel.*.
USING WebApp.System.Session.*.
USING WebApp.System.Manager.*.
USING WebApp.System.Util.*.
USING WebApp.EC.Util.*.


BLOCK-LEVEL ON ERROR UNDO, THROW.

CLASS WebApp.EC.BL.ContainerBE
      INHERITS WebApp.System.BusinessLogic.BusinessEntity:

  {"WebApp/EC/BL/Container.i"}

  DEFINE DATA-SOURCE srcContainer FOR ec-container.

  /*-------------------------------------------------------- ----------------------
          Purpose: Prepare the dataset and skipped field lists.
          Notes:
  ------------------------------------------------------------------------------*/
  CONSTRUCTOR PUBLIC ContainerBE():

    DEFINE VAR hDataSourceArray AS HANDLE    NO-UNDO EXTENT 1.
    DEFINE VAR cSkipListArray   AS CHARACTER NO-UNDO EXTENT 1.
    DEFINE VAR lUseCompanyScope AS LOGICAL   NO-UNDO EXTENT 1.
    DEFINE VAR cUseIndex        AS CHARACTER NO-UNDO EXTENT 1.

    SUPER(DATASET dsContainer:HANDLE).

    /* Data Source for each table in dataset. Should be in table order as defined in DataSet */
    hDataSourceArray[1] = DATA-SOURCE srcContainer:HANDLE.

    /* Skip-list entry for each table in dataset. Should be in temp-table order as defined in DataSet */
    /* Each skip-list entry is a comma-separated list of field names, to be ignored in create statement */
    cSkipListArray[1] = ?.

    /* Then 'companycode' will automaticly addded to the Where Clause by default and save on submit
       Use lUseCompanyScope = NO to ignore this behavour */
    lUseCompanyScope[1] = ?.
    cUseIndex[1] = ?.

    THIS-OBJECT:ProDataSource     = hDataSourceArray.
    THIS-OBJECT:SkipList          = cSkipListArray.
    THIS-OBJECT:spUseCompanyScope = lUseCompanyScope.
    THIS-OBJECT:spUseIndex        = cUseIndex.

    

    THIS-OBJECT:Validator         = NEW WebApp.EC.BL.ContainerValidator().
    THIS-OBJECT:UIController      = NEW WebApp.EC.BL.ContainerUIController().

    THIS-OBJECT:FilterTypes = "default,ablFilter,kendoFilter".

  END CONSTRUCTOR.

  /*------------------------------------------------------------------------------
    Purpose:  Define calculated fields
    Notes:    @param phBuffer buffer to current record
  ------------------------------------------------------------------------------*/
  METHOD OVERRIDE PROTECTED VOID DataCalculate( INPUT phBuffer AS HANDLE  ):


    /* Softpak Converted Code Start */

    phBuffer::calc_a_1_last_vessel_move = ecLastVesselMove( phBuffer ).
    phBuffer::calc_dt_onhire = date(ecLastContract('dt_contract')).
    phBuffer::calc_z_discharge_date = ecGetLastVesselDetails('dt_move','','').
    phBuffer::calc_z_discharge_location = ecGetLastVesselDetails('locationcode','','').
    phBuffer::calc_z_idle_days = if phBuffer::act_dt_move < today then today - phBuffer::act_dt_move else 0.
    phBuffer::calc_z_lastvessel = ecGetLastVesselDetails('vesselcode','ec-vessel','vesselcode+bookingnr+des').
    phBuffer::calc_z_lastvesselcode = ecGetLastVesselDetails('vesselcode','ec-vessel','vesselcode').
    phBuffer::calc_z_onhire_days = if phBuffer::dt_onhire < today then today - phBuffer::dt_onhire else 0.
    phBuffer::xref_consignee = ecGetShipmentDetails('CONSIGNEE',phBuffer::bookingnr).
    phBuffer::xref_container = phBuffer::containerprefix + " " + STRING(phBuffer::containernr,"XXXXXX/X").
    phBuffer::xref_contract = ecLastContract('contractcode', phBuffer ).
    XrefHelper:spXref("aa-address", "addressnr"  , "customer", "name_1", phBuffer, "xref_customer").
    phBuffer::xref_leasingcompany = ecLastContract('leasingcompany').
    phBuffer::xref_off_hire_location = ''.
    phBuffer::xref_off_hire_reference = ''.
    phBuffer::xref_shipper = ecGetShipmentDetails('SHIPPER',phBuffer::bookingnr).
    phBuffer::xref_voyage = ecgetVoyage(phBuffer::act_vesselcode).

    /* Softpak Converted Code End */


  END METHOD.

  /*------------------------------------------------------------------------------
     Purpose: Get total count of records, based on a filter string.
     Notes:   @param filter Filter reseved from front-end
              @param numRecs Number of records in this table
              @param exact result count exact or guess (count maximum 1 seconds)
  ------------------------------------------------------------------------------*/
  @openapi.openedge.export(type="REST", useReturnValue="false", writeDataSetBeforeImage="false").
  @progress.service.resourceMapping(type="REST", operation="invoke", URI="/count?filter=~{filter~}", alias="count", mediaType="application/json").
  METHOD PUBLIC VOID CountRecords( INPUT filter AS CHARACTER, OUTPUT numRecs AS INTEGER, OUTPUT exact AS LOGICAL):

    SUPER:Count(INPUT filter, OUTPUT numRecs, OUTPUT exact).

  END METHOD. /* CountRecords */

  /*------------------------------------------------------------------------------
   Purpose: Combine Field sensitivity, values and mandatory fields in a single call from the frontend
   Notes: @param pcInvokeField invloked field
          @param dsContainer prodataset received from front-end
          @param dsFieldState prodataset with mandatory fields & UI Control (Sensitivity & Field Values)
  ------------------------------------------------------------------------------*/
  @openapi.openedge.export(type="REST", useReturnValue="false", writeDataSetBeforeImage="true").
  @progress.service.resourceMapping(type="REST", operation="invoke", URI="/fieldState", alias="fieldState", mediaType="application/json").
  METHOD PUBLIC VOID FieldStateContainer( INPUT  invokeField AS CHARACTER, INPUT newRow AS LOGICAL, INPUT-OUTPUT DATASET dsContainer, OUTPUT DATASET dsFieldState ):


    DEFINE VARIABLE hDataSet AS HANDLE NO-UNDO.
    ASSIGN hDataSet = DATASET dsContainer:HANDLE.

    TEMP-TABLE ttContainer:TRACKING-CHANGES = TRUE.

    SUPER:FieldState(INPUT invokeField,
                     INPUT newRow,
                     INPUT-OUTPUT DATASET-HANDLE hDataSet,
                     OUTPUT DATASET dsFieldState ).

    FINALLY:
      TEMP-TABLE ttContainer:TRACKING-CHANGES = FALSE.
    END FINALLY.

  END METHOD.


  /*------------------------------------------------------------------------------
    Purpose: Get one or more records, based on a filter string
    Notes: @param filter Filter received from font-end.
           @param  dsContainer  proDataset with result of request
  ------------------------------------------------------------------------------*/
  @openapi.openedge.export(type="REST", useReturnValue="false", writeDataSetBeforeImage="true").
  @progress.service.resourceMapping(type="REST", operation="read", URI="?filter=~{filter~}", alias="", mediaType="application/json").
  @openapi.openedge.method.property(name="mappingType", value="SPJFP").
  @openapi.openedge.method.property(name="capabilities",value="kendoFilter,top,skip,id,orderBy,contextKey").
  METHOD PUBLIC VOID ReadContainer ( INPUT filter AS CHARACTER, OUTPUT DATASET dsContainer ):

    SUPER:ReadData(filter).
    THIS-OBJECT:PostFetchRecord(filter).

  END METHOD. /* Read */

  /*------------------------------------------------------------------------------
    Purpose: Submit one or more records (with create, update, delete action).
    Notes: @param dsContainer proDataset to submit
  ------------------------------------------------------------------------------*/
  @openapi.openedge.export(type="REST", useReturnValue="false", writeDataSetBeforeImage="true").
  @progress.service.resourceMapping(type="REST", operation="submit", URI="/submit", alias="submit", mediaType="application/json").
  METHOD PUBLIC VOID SubmitContainer ( INPUT-OUTPUT DATASET dsContainer ):

    DEFINE VARIABLE hDataSet AS HANDLE NO-UNDO.
    ASSIGN hDataSet = DATASET dsContainer:HANDLE.

    IF NOT THIS-OBJECT:ValidateData   (INPUT DATASET dsContainer BY-REFERENCE) THEN LEAVE.
    IF NOT THIS-OBJECT:PreCommitLogic (INPUT DATASET dsContainer BY-REFERENCE) THEN LEAVE.

    /* USE BY-REFERENCE https://knowledgebase.progress.com/articles/Article/businessentity-not-returning-prods-clientid */
    SUPER:Submit(INPUT-OUTPUT DATASET-HANDLE hDataSet BY-REFERENCE).

    IF NOT THIS-OBJECT:PostCommitLogic(INPUT DATASET dsContainer BY-REFERENCE) THEN LEAVE.

  END METHOD. /* Submit */

  /*------------------------------------------------------------------------------
          Purpose: Provide defaults for a new record.
          Notes: @param dsContainer proDataset with new records to create
  ------------------------------------------------------------------------------*/
  @openapi.openedge.export(type="REST", useReturnValue="false", writeDataSetBeforeImage="false").
  @progress.service.resourceMapping(type="REST", operation="invoke", URI="/new", alias="new", mediaType="application/json").
  METHOD PUBLIC VOID NewRecord ( INPUT-OUTPUT DATASET dsContainer ):

  END METHOD. /* NewRecord */

  /************************************************************************************************/
  /***************************  Business Entity specific methods **********************************/
  /************************************************************************************************/
  /*------------------------------------------------------------------------------
    Purpose: Validate data before create, update, or delete.
    Notes:  @param dsContainer proDataset to validate
    ------------------------------------------------------------------------------*/
  METHOD PROTECTED LOGICAL ValidateData (INPUT DATASET dsContainer):

   DEFINE VARIABLE lErrors    AS LOGICAL NO-UNDO.
   DEFINE VARIABLE oValidator AS WebApp.EC.BL.ContainerValidator NO-UNDO.

   IF NOT CAN-FIND(ttContainer)  AND NOT CAN-FIND(bttContainer)
   THEN LEAVE.

   oValidator = CAST(THIS-OBJECT:Validator, WebApp.EC.BL.ContainerValidator).
   oValidator:Validate(INPUT DATASET dsContainer BY-REFERENCE, OUTPUT lErrors).

   RETURN NOT lErrors. /* Succeed */

  END METHOD. /* ValidateData */

  /*------------------------------------------------------------------------------
    Purpose: Any PreCommit logic for create, update, or delete.
    Notes:
    ------------------------------------------------------------------------------*/
  METHOD PROTECTED LOGICAL PreCommitLogic ( INPUT DATASET dsContainer ):

  END METHOD. /* PreCommitLogic*/

  /*------------------------------------------------------------------------------
    Purpose:  Any PostCommit logic for create, update, or delete.
    Notes:
  ------------------------------------------------------------------------------*/
  METHOD PROTECTED LOGICAL PostCommitLogic ( INPUT DATASET dsContainer):

  END METHOD. /* PostCommitLogic */

  /*------------------------------------------------------------------------------
    Purpose: Perform any post-read logic.
    Notes:
  ------------------------------------------------------------------------------*/
  METHOD PROTECTED VOID PostFetchRecord ( INPUT filter AS CHARACTER ):

  END METHOD. /* PostFetchRecord */


  /************************************************************************************************/
  /*************************************  Custom Methods  *****************************************/
  /************************************************************************************************/
  /* Softpak Converted Code Start */

  /*------------------------------------------------------------------------------
    Purpose:
      Notes:
  ------------------------------------------------------------------------------*/
  METHOD PRIVATE INT ecLastVesselMove
    ( INPUT phBuffer AS HANDLE ) :

    DEF VAR lRead                     AS LOGICAL   NO-UNDO.

    giLastVesselMove    = 0.
    giLastDischargeMove = 0.
    gcLastVesselDetails = "".
    FIND LAST ec-container_history WHERE
              ec-container_history.companycode     = SpSession:Companycode
          AND ec-container_history.containerprefix = phBuffer::containerprefix
          AND ec-container_history.containernr     = phBuffer::containernr
          AND LOOKUP(ec-container_history.statuscode,gcDischargeStatusList,CHR(1)) > 0
              USE-INDEX dt_move NO-LOCK NO-ERROR.
    IF AVAIL ec-container_history
    THEN DO:
      giLastDischargeMove = ec-container_history.MOVE_refnr.
      gcLastVesselDetails = 'locationcode' + CHR(3) + ec-container_history.locationcode
                          + CHR(3) + 'dt_move' + CHR(3) + STRING(ec-container_history.dt_move,"99/99/9999").

      lRead = TRUE.
      DO WHILE lRead:
        FIND PREV ec-container_history WHERE
                  ec-container_history.companycode     = SpSession:Companycode
              AND ec-container_history.containerprefix = phBuffer::containerprefix
              AND ec-container_history.containernr     = phBuffer::containernr
                  USE-INDEX dt_move NO-LOCK NO-ERROR.
        IF NOT AVAIL ec-container_history
        THEN lRead = FALSE.
        ELSE DO:
          IF LOOKUP(ec-container_history.statuscode,gcLoadedStatusList,CHR(1)) > 0
          THEN lRead = FALSE.
        END.
      END.

      IF AVAIL ec-container_history
      THEN DO:
        giLastVesselMove    = ec-container_history.MOVE_refnr.
        gcLastVesselDetails = gcLastVesselDetails
                            + CHR(3) + 'statuscode'    + CHR(3) + ec-container_history.statuscode
                            + CHR(3) + 'vesselcode'    + CHR(3) + ec-container_history.vesselcode
                            + CHR(3) + 'pol_portcode'  + CHR(3) + ec-container_history.pol_portcode
                            + CHR(3) + 'pod_portcode'  + CHR(3) + ec-container_history.pod_portcode
                            + CHR(3) + 'dest_portcode' + CHR(3) + ec-container_history.dest_portcode
                            + CHR(3) + 'blnr'          + CHR(3) + ec-container_history.blnr
                            + CHR(3) + 'bookingnr'     + CHR(3) + ec-container_history.bookingnr
                            + CHR(3) + 'customer'      + CHR(3) + STRING(ec-container_history.customer)
                            + CHR(3) + 'servicecode'   + CHR(3) + ec-container_history.servicecode
                            + CHR(3) + 'weight'        + CHR(3) + STRING(ec-container_history.weight)
                            + CHR(3) + 'stow_cell'     + CHR(3) + ec-container_history.stow_cell.
      END.
      ELSE DO:
        giLastVesselMove    = 0.
        giLastDischargeMove = 0.
        gcLastVesselDetails = "".
      END.
    END.

    RETURN giLastVesselMove.

  END METHOD.

  /*------------------------------------------------------------------------------
    Purpose:
      Notes:
  ------------------------------------------------------------------------------*/
  METHOD PRIVATE CHARACTER ecGetLastVesselDetails
    ( INPUT pcField     AS CHAR
     ,INPUT pcXrefTable AS CHAR
     ,INPUT pcXrefField AS CHAR ) :

    DEF VAR cReturnValue              AS CHAR      NO-UNDO.
    DEF VAR hBuffer                   AS HANDLE    NO-UNDO.
    DEF VAR hField                    AS HANDLE    NO-UNDO.
    DEF VAR iLoop                     AS INTEGER   NO-UNDO.

    cReturnValue = spGetEntry(pcField,gcLastVesselDetails,CHR(3),CHR(3)).
    IF pcXrefTable <> ""
    THEN DO:
      CASE pcXrefTable:
        WHEN 'ec-vessel'
        THEN DO:
          FIND ec-vessel WHERE
               ec-vessel.companycode = SpSession:Companycode
           AND ec-vessel.vesselcode  = cReturnValue
               NO-LOCK NO-ERROR.
          IF AVAIL ec-vessel
          THEN hBuffer = BUFFER ec-vessel:HANDLE.
        END.
        WHEN 'aa-port'
        THEN DO:
          FIND aa-port WHERE
               aa-port.portcode  = cReturnValue
               NO-LOCK NO-ERROR.
          IF AVAIL aa-port
          THEN hBuffer = BUFFER aa-port:HANDLE.
        END.
        WHEN 'aa-address'
        THEN DO:
          FIND aa-address WHERE
               aa-address.addressnr  = int(cReturnValue)
               NO-LOCK NO-ERROR.
          IF AVAIL aa-address
          THEN hBuffer = BUFFER aa-address:HANDLE.
        END.
        WHEN 'aa-service'
        THEN DO:
          FIND aa-service WHERE
               aa-service.companycode = SpSession:Companycode
           AND aa-service.servicecode  = cReturnValue
               NO-LOCK NO-ERROR.
          IF AVAIL aa-service
          THEN hBuffer = BUFFER aa-service:HANDLE.
        END.
      END.

      cReturnValue = "".
      DO iLoop = 1 TO NUM-ENTRIES(pcXrefField,"+"):
        hField = ?.
        IF VALID-HANDLE(hBuffer)
        THEN hField = hBuffer:BUFFER-FIELD(ENTRY(iLoop,pcXrefField,"+")) NO-ERROR.
        IF VALID-HANDLE(hField)
        THEN cReturnValue = cReturnValue
                          + (IF cReturnValue = "" THEN "" ELSE " ")
                          + hField:BUFFER-VALUE.
        ELSE cReturnValue = cReturnValue
                          + (IF cReturnValue = "" THEN "" ELSE " ")
                          + spGetEntry(ENTRY(iLoop,pcXrefField,"+"),gcLastVesselDetails,CHR(3),CHR(3)).
      END.
      IF VALID-HANDLE(hBuffer)
      THEN DELETE OBJECT hBuffer.
    END.

    RETURN cReturnValue.

  END METHOD.

  /*------------------------------------------------------------------------------
    Purpose:
      Notes:
  ------------------------------------------------------------------------------*/
  METHOD PRIVATE CHARACTER ecGetShipmentDetails
    ( INPUT pcMode      AS CHAR
     ,INPUT pcBookingnr AS CHAR) :

    {ec/include/ec-get-shipment-details.i}

  END METHOD.

  /*------------------------------------------------------------------------------
    Purpose:
      Notes:
  ------------------------------------------------------------------------------*/
  METHOD PRIVATE CHARACTER ecLastContract
    ( INPUT pcField AS CHAR, INPUT phBuffer AS HANDLE ) :

    DEF VAR cReturnValue              AS CHARACTER NO-UNDO.

    FIND LAST ec-container_contract WHERE
              ec-container_contract.companycode     = SpSession:Companycode
          AND ec-container_contract.containerprefix = phBuffer::containerprefix
          AND ec-container_contract.containernr     = phBuffer::containernr
              USE-INDEX ec-container_contract NO-LOCK NO-ERROR.
    IF AVAIL ec-container_contract
    THEN DO:
      CASE pcField:
        WHEN 'contractcode' THEN cReturnValue = ec-container_contract.contractcode.
        WHEN 'dt_contract'  THEN cReturnValue = STRING(ec-container_contract.dt_contract).
        WHEN 'leasingcompany'
        THEN DO:
          FIND FIRST ec-contract WHERE
                     ec-contract.companycode = SpSession:Companycode
                 AND ec-contract.contractcode = ec-container_contract.contractcode NO-LOCK NO-ERROR.
          IF AVAILABLE ec-contract
          THEN DO:
            FIND FIRST aa-address WHERE
                       aa-address.addressnr = ec-contract.lease_addressnr NO-LOCK NO-ERROR.
            IF AVAILABLE aa-address THEN cReturnValue = aa-address.name_1.
          END.
        END.
      END CASE.
    END.

    RETURN cReturnValue.

  END METHOD.

  /*------------------------------------------------------------------------------
    Purpose:
      Notes:
  ------------------------------------------------------------------------------*/
  METHOD PRIVATE CHARACTER ecgetVoyage
    ( INPUT pcVessel AS CHAR ) :

    DEF VAR cReturnValue AS CHAR NO-UNDO.

    IF pcVessel <> ""
    THEN DO:
      FIND ec-vessel WHERE
           ec-vessel.companycode = SpSession:Companycode
       AND ec-vessel.vesselcode  = pcVessel
           NO-LOCK NO-ERROR.

      IF AVAIL ec-vessel
      THEN ASSIGN cReturnValue = ec-vessel.voyage + " " + ec-vessel.des.
    END.

    RETURN cReturnValue.

  END METHOD.

  /* Softpak Converted Code End */


END CLASS.
